@@ -1,10 +1,8 @@
 #' \pkg{dplyr} table manipulation methods for zoomed dm objects
 #'
 #' @description
-#' `r lifecycle::badge("stable")`
-#'
-#' Use these methods without the '.zoomed_dm' suffix (see examples).
-#' @param .data object of class `zoomed_dm`
+#' Use these methods without the '.dm_zoomed' suffix (see examples).
+#' @param .data object of class `dm_zoomed`
 #' @param ... see corresponding function in package \pkg{dplyr} or \pkg{tidyr}
 #' @name dplyr_table_manipulation
 #' @examplesIf rlang::is_installed("nycflights13")
@@ -24,7 +22,7 @@ filter.dm <- function(.data, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-filter.zoomed_dm <- function(.data, ...) {
+filter.dm_zoomed <- function(.data, ...) {
   .data %>%
     dm_filter_impl(..., set_filter = FALSE)
 }
@@ -36,7 +34,7 @@ mutate.dm <- function(.data, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-mutate.zoomed_dm <- function(.data, ...) {
+mutate.dm_zoomed <- function(.data, ...) {
   tbl <- tbl_zoomed(.data)
   mutated_tbl <- mutate(tbl, ...)
   # #663: user responsibility: those columns are tracked whose names remain
@@ -52,7 +50,7 @@ transmute.dm <- function(.data, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-transmute.zoomed_dm <- function(.data, ...) {
+transmute.dm_zoomed <- function(.data, ...) {
   tbl <- tbl_zoomed(.data)
   # groups are "selected"; key tracking will continue for them
   groups <- set_names(map_chr(groups(tbl), as_string))
@@ -72,7 +70,7 @@ select.dm <- function(.data, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-select.zoomed_dm <- function(.data, ...) {
+select.dm_zoomed <- function(.data, ...) {
   tbl <- tbl_zoomed(.data)
 
   selected <- eval_select_both(quo(c(...)), colnames(tbl))
@@ -90,7 +88,7 @@ relocate.dm <- function(.data, ...) {
 #' @rdname dplyr_table_manipulation
 #' @inheritParams dplyr::relocate
 #' @export
-relocate.zoomed_dm <- function(.data, ..., .before = NULL, .after = NULL) {
+relocate.dm_zoomed <- function(.data, ..., .before = NULL, .after = NULL) {
   tbl <- tbl_zoomed(.data)
 
   relocated_tbl <- relocate(tbl, ..., .before = {{ .before }}, .after = {{ .after }})
@@ -104,7 +102,7 @@ rename.dm <- function(.data, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-rename.zoomed_dm <- function(.data, ...) {
+rename.dm_zoomed <- function(.data, ...) {
   tbl <- tbl_zoomed(.data)
 
   renamed <- eval_rename_both(quo(c(...)), colnames(tbl))
@@ -120,9 +118,9 @@ distinct.dm <- function(.data, ...) {
 }
 
 #' @rdname dplyr_table_manipulation
-#' @param .keep_all For `distinct.zoomed_dm()`: see [`dplyr::distinct`]
+#' @param .keep_all For `distinct.dm_zoomed()`: see [dplyr::distinct()]
 #' @export
-distinct.zoomed_dm <- function(.data, ..., .keep_all = FALSE) {
+distinct.dm_zoomed <- function(.data, ..., .keep_all = FALSE) {
   tbl <- tbl_zoomed(.data)
   distinct_tbl <- distinct(tbl, ..., .keep_all = .keep_all)
 
@@ -140,7 +138,7 @@ arrange.dm <- function(.data, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-arrange.zoomed_dm <- function(.data, ...) {
+arrange.dm_zoomed <- function(.data, ...) {
   tbl <- tbl_zoomed(.data)
   arranged_tbl <- arrange(tbl, ...)
   replace_zoomed_tbl(.data, arranged_tbl)
@@ -152,11 +150,11 @@ slice.dm <- function(.data, ...) {
 }
 
 #' @rdname dplyr_table_manipulation
-#' @param .keep_pk For `slice.zoomed_dm`: Logical, if `TRUE`, the primary key will be retained during this transformation. If `FALSE`, it will be dropped.
+#' @param .keep_pk For `slice.dm_zoomed`: Logical, if `TRUE`, the primary key will be retained during this transformation. If `FALSE`, it will be dropped.
 #' By default, the value is `NULL`, which causes the function to issue a message in case a primary key is available for the zoomed table.
-#' This argument is specific for the `slice.zoomed_dm()` method.
+#' This argument is specific for the `slice.dm_zoomed()` method.
 #' @export
-slice.zoomed_dm <- function(.data, ..., .keep_pk = NULL) {
+slice.dm_zoomed <- function(.data, ..., .keep_pk = NULL) {
   sliced_tbl <- slice(tbl_zoomed(.data), ...)
   orig_pk <- dm_get_pk_impl(.data, orig_name_zoomed(.data))
   tracked_cols <- col_tracker_zoomed(.data)
@@ -164,7 +162,7 @@ slice.zoomed_dm <- function(.data, ..., .keep_pk = NULL) {
     if (has_length(orig_pk) && any(unlist(orig_pk) %in% tracked_cols)) {
       message(
         paste(
-          "Keeping PK column, but `slice.zoomed_dm()` can potentially damage the uniqueness of PK columns (duplicated indices).",
+          "Keeping PK column, but `slice.dm_zoomed()` can potentially damage the uniqueness of PK columns (duplicated indices).",
           "Set argument `.keep_pk` to `TRUE` or `FALSE` to ensure the behavior you intended."
         )
       )
@@ -182,20 +180,29 @@ group_by.dm <- function(.data, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-group_by.zoomed_dm <- function(.data, ...) {
+group_by.dm_zoomed <- function(.data, ...) {
   tbl <- tbl_zoomed(.data)
   grouped_tbl <- group_by(tbl, ...)
 
   replace_zoomed_tbl(.data, grouped_tbl)
 }
 
+#' @rdname dplyr_table_manipulation
+#' @export
+group_by.dm_keyed_tbl <- function(.data, ...) {
+  keys_info <- keyed_get_info(.data)
+  tbl <- unclass_keyed_tbl(.data)
+  grouped_tbl <- group_by(tbl, ...)
+  new_keyed_tbl_from_keys_info(grouped_tbl, keys_info)
+}
+
 #' @export
 group_data.dm <- function(.data) {
   check_zoomed(.data)
 }
 
 #' @export
-group_data.zoomed_dm <- function(.data) {
+group_data.dm_zoomed <- function(.data) {
   tbl <- tbl_zoomed(.data)
   group_data(tbl)
 }
@@ -206,7 +213,7 @@ group_keys.dm <- function(.tbl, ...) {
 }
 
 #' @export
-group_keys.zoomed_dm <- function(.tbl, ...) {
+group_keys.dm_zoomed <- function(.tbl, ...) {
   .data <- .tbl
   tbl <- tbl_zoomed(.data)
   group_keys(tbl, ...)
@@ -218,7 +225,7 @@ group_indices.dm <- function(.data, ...) {
 }
 
 #' @export
-group_indices.zoomed_dm <- function(.data, ...) {
+group_indices.dm_zoomed <- function(.data, ...) {
   tbl <- tbl_zoomed(.data)
   group_indices(tbl, ...)
 }
@@ -229,7 +236,7 @@ group_vars.dm <- function(x) {
 }
 
 #' @export
-group_vars.zoomed_dm <- function(x) {
+group_vars.dm_zoomed <- function(x) {
   .data <- x
   tbl <- tbl_zoomed(.data)
   group_vars(tbl)
@@ -241,7 +248,7 @@ groups.dm <- function(x) {
 }
 
 #' @export
-groups.zoomed_dm <- function(x) {
+groups.dm_zoomed <- function(x) {
   .data <- x
   tbl <- tbl_zoomed(.data)
   groups(tbl)
@@ -253,9 +260,9 @@ ungroup.dm <- function(x, ...) {
 }
 
 #' @rdname dplyr_table_manipulation
-#' @param x For `ungroup.zoomed_dm`: object of class `zoomed_dm`
+#' @param x For `ungroup.dm_zoomed`: object of class `dm_zoomed`
 #' @export
-ungroup.zoomed_dm <- function(x, ...) {
+ungroup.dm_zoomed <- function(x, ...) {
   tbl <- tbl_zoomed(x)
   ungrouped_tbl <- ungroup(tbl, ...)
 
@@ -269,7 +276,7 @@ summarise.dm <- function(.data, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-summarise.zoomed_dm <- function(.data, ...) {
+summarise.dm_zoomed <- function(.data, ...) {
   tbl <- tbl_zoomed(.data)
   # groups are "selected"; key tracking will continue for them
   # #663: user responsibility: if group columns are manipulated, they are still tracked
@@ -279,6 +286,29 @@ summarise.zoomed_dm <- function(.data, ...) {
   replace_zoomed_tbl(.data, summarized_tbl, new_tracked_cols_zoom)
 }
 
+
+#' @rdname dplyr_table_manipulation
+#' @export
+summarise.dm_keyed_tbl <- function(.data, ...) {
+  keys_info <- keyed_get_info(.data)
+  tbl <- unclass_keyed_tbl(.data)
+
+  new_pk <- group_vars(.data)
+  if (length(new_pk) == 0) {
+    new_pk <- NULL
+  }
+
+  summarised_tbl <- NextMethod()
+
+  # FIXME: Add original FKs for the remaining columns
+  # (subsets of the grouped columns), use new UUID
+  new_keyed_tbl(
+    summarised_tbl,
+    pk = new_pk,
+    uuid = keys_info$uuid
+  )
+}
+
 #' @export
 count.dm <- function(x, ...) {
   check_zoomed(x)
@@ -287,7 +317,7 @@ count.dm <- function(x, ...) {
 #' @rdname dplyr_table_manipulation
 #' @inheritParams dplyr::count
 #' @export
-count.zoomed_dm <- function(x, ..., wt = NULL, sort = FALSE, name = NULL,
+count.dm_zoomed <- function(x, ..., wt = NULL, sort = FALSE, name = NULL,
                             .drop = group_by_drop_default(x)) {
   tbl <- tbl_zoomed(x)
 
@@ -317,7 +347,7 @@ tally.dm <- function(x, ...) {
 
 #' @rdname dplyr_table_manipulation
 #' @export
-tally.zoomed_dm <- function(x, ...) {
+tally.dm_zoomed <- function(x, ...) {
   tbl <- tbl_zoomed(x)
   groups <- set_names(map_chr(groups(tbl), as_string))
 
@@ -340,32 +370,32 @@ pull.dm <- function(.data, var = -1, name = NULL) {
 #' @rdname dplyr_table_manipulation
 #' @inheritParams dplyr::pull
 #' @export
-pull.zoomed_dm <- function(.data, var = -1, ...) {
+pull.dm_zoomed <- function(.data, var = -1, ...) {
   tbl <- tbl_zoomed(.data)
   pull(tbl, var = {{ var }}, ...)
 }
 
 #' @rdname dplyr_table_manipulation
 #' @export
-compute.zoomed_dm <- function(x, ...) {
-  zoomed_df <-
+compute.dm_zoomed <- function(x, ...) {
+  dm_zoomed_df <-
     tbl_zoomed(x) %>%
     compute(...)
-  replace_zoomed_tbl(x, zoomed_df)
+  replace_zoomed_tbl(x, dm_zoomed_df)
 }
 
 #' \pkg{dplyr} join methods for zoomed dm objects
 #'
-#' Use these methods without the '.zoomed_dm' suffix (see examples).
+#' Use these methods without the '.dm_zoomed' suffix (see examples).
 #' @name dplyr_join
-#' @param x,y tbls to join. `x` is the `zoomed_dm` and `y` is another table in the `dm`.
+#' @param x,y tbls to join. `x` is the `dm_zoomed` and `y` is another table in the `dm`.
 #' @param by If left `NULL` (default), the join will be performed by via the foreign key relation that exists between the originally zoomed table (now `x`)
 #' and the other table (`y`).
 #' If you provide a value (for the syntax see [`dplyr::join`]), you can also join tables that are not connected in the `dm`.
 #' @param copy Disabled, since all tables in a `dm` are by definition on the same `src`.
 #' @param suffix Disabled, since columns are disambiguated automatically if necessary, changing the column names to `table_name.column_name`.
 #' @param select Select a subset of the \strong{RHS-table}'s columns, the syntax being `select = c(col_1, col_2, col_3)` (unquoted or quoted).
-#' This argument is specific for the `join`-methods for `zoomed_dm`.
+#' This argument is specific for the `join`-methods for `dm_zoomed`.
 #' The table's `by` column(s) are automatically added if missing in the selection.
 #' @param ... see [`dplyr::join`]
 #' @examplesIf rlang::is_installed("nycflights13")
@@ -385,13 +415,38 @@ left_join.dm <- function(x, ...) {
 
 #' @rdname dplyr_join
 #' @export
-left_join.zoomed_dm <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
+left_join.dm_zoomed <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
   y_name <- as_string(enexpr(y))
   join_data <- prepare_join(x, {{ y }}, by, {{ select }}, suffix, copy)
   joined_tbl <- left_join(join_data$x_tbl, join_data$y_tbl, join_data$by, copy = FALSE, ...)
   replace_zoomed_tbl(x, joined_tbl, join_data$new_col_names)
 }
 
+#' @rdname dplyr_join
+#' @export
+left_join.dm_keyed_tbl <- function(x, y, by = NULL, copy = NULL, suffix = NULL, ..., keep = FALSE) {
+  if (!is_dm_keyed_tbl(y)) {
+    return(NextMethod())
+  }
+
+  join_spec <- keyed_build_join_spec(x, y, by, suffix)
+  joined_tbl <- left_join(
+    join_spec$x_tbl, join_spec$y_tbl, deframe(join_spec$by),
+    copy = copy,
+    suffix = join_spec$suffix,
+    keep = keep,
+    ...
+  )
+
+  new_keyed_tbl(
+    joined_tbl,
+    pk = join_spec$new_pk,
+    fks_in = join_spec$new_fks_in,
+    fks_out = join_spec$new_fks_out,
+    uuid = join_spec$new_uuid
+  )
+}
+
 #' @export
 inner_join.dm <- function(x, ...) {
   check_zoomed(x)
@@ -399,13 +454,38 @@ inner_join.dm <- function(x, ...) {
 
 #' @rdname dplyr_join
 #' @export
-inner_join.zoomed_dm <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
+inner_join.dm_zoomed <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
   y_name <- as_string(enexpr(y))
   join_data <- prepare_join(x, {{ y }}, by, {{ select }}, suffix, copy)
   joined_tbl <- inner_join(join_data$x_tbl, join_data$y_tbl, join_data$by, copy = FALSE, ...)
   replace_zoomed_tbl(x, joined_tbl, join_data$new_col_names)
 }
 
+#' @rdname dplyr_join
+#' @export
+inner_join.dm_keyed_tbl <- function(x, y, by = NULL, copy = NULL, suffix = NULL, ..., keep = FALSE) {
+  if (!is_dm_keyed_tbl(y)) {
+    return(NextMethod())
+  }
+
+  join_spec <- keyed_build_join_spec(x, y, by, suffix)
+  joined_tbl <- inner_join(
+    join_spec$x_tbl, join_spec$y_tbl, deframe(join_spec$by),
+    copy = copy,
+    suffix = join_spec$suffix,
+    keep = keep,
+    ...
+  )
+
+  new_keyed_tbl(
+    joined_tbl,
+    pk = join_spec$new_pk,
+    fks_in = join_spec$new_fks_in,
+    fks_out = join_spec$new_fks_out,
+    uuid = join_spec$new_uuid
+  )
+}
+
 #' @export
 full_join.dm <- function(x, ...) {
   check_zoomed(x)
@@ -413,13 +493,38 @@ full_join.dm <- function(x, ...) {
 
 #' @rdname dplyr_join
 #' @export
-full_join.zoomed_dm <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
+full_join.dm_zoomed <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
   y_name <- as_string(enexpr(y))
   join_data <- prepare_join(x, {{ y }}, by, {{ select }}, suffix, copy)
   joined_tbl <- full_join(join_data$x_tbl, join_data$y_tbl, join_data$by, copy = FALSE, ...)
   replace_zoomed_tbl(x, joined_tbl, join_data$new_col_names)
 }
 
+#' @rdname dplyr_join
+#' @export
+full_join.dm_keyed_tbl <- function(x, y, by = NULL, copy = NULL, suffix = NULL, ..., keep = FALSE) {
+  if (!is_dm_keyed_tbl(y)) {
+    return(NextMethod())
+  }
+
+  join_spec <- keyed_build_join_spec(x, y, by, suffix)
+  joined_tbl <- full_join(
+    join_spec$x_tbl, join_spec$y_tbl, deframe(join_spec$by),
+    copy = copy,
+    suffix = join_spec$suffix,
+    keep = keep,
+    ...
+  )
+
+  new_keyed_tbl(
+    joined_tbl,
+    pk = join_spec$new_pk,
+    fks_in = join_spec$new_fks_in,
+    fks_out = join_spec$new_fks_out,
+    uuid = join_spec$new_uuid
+  )
+}
+
 #' @export
 right_join.dm <- function(x, ...) {
   check_zoomed(x)
@@ -427,13 +532,38 @@ right_join.dm <- function(x, ...) {
 
 #' @rdname dplyr_join
 #' @export
-right_join.zoomed_dm <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
+right_join.dm_zoomed <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
   y_name <- as_string(enexpr(y))
   join_data <- prepare_join(x, {{ y }}, by, {{ select }}, suffix, copy)
   joined_tbl <- right_join(join_data$x_tbl, join_data$y_tbl, join_data$by, copy = FALSE, ...)
   replace_zoomed_tbl(x, joined_tbl, join_data$new_col_names)
 }
 
+#' @rdname dplyr_join
+#' @export
+right_join.dm_keyed_tbl <- function(x, y, by = NULL, copy = NULL, suffix = NULL, ..., keep = FALSE) {
+  if (!is_dm_keyed_tbl(y)) {
+    return(NextMethod())
+  }
+
+  join_spec <- keyed_build_join_spec(x, y, by, suffix)
+  joined_tbl <- right_join(
+    join_spec$x_tbl, join_spec$y_tbl, deframe(join_spec$by),
+    copy = copy,
+    suffix = join_spec$suffix,
+    keep = keep,
+    ...
+  )
+
+  new_keyed_tbl(
+    joined_tbl,
+    pk = join_spec$new_pk,
+    fks_in = join_spec$new_fks_in,
+    fks_out = join_spec$new_fks_out,
+    uuid = join_spec$new_uuid
+  )
+}
+
 #' @export
 semi_join.dm <- function(x, ...) {
   check_zoomed(x)
@@ -441,13 +571,33 @@ semi_join.dm <- function(x, ...) {
 
 #' @rdname dplyr_join
 #' @export
-semi_join.zoomed_dm <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
+semi_join.dm_zoomed <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
   y_name <- as_string(enexpr(y))
   join_data <- prepare_join(x, {{ y }}, by, {{ select }}, suffix, copy, disambiguate = FALSE)
   joined_tbl <- semi_join(join_data$x_tbl, join_data$y_tbl, join_data$by, copy = FALSE, ...)
   replace_zoomed_tbl(x, joined_tbl, join_data$new_col_names)
 }
 
+#' @rdname dplyr_join
+#' @export
+semi_join.dm_keyed_tbl <- function(x, y, by = NULL, copy = NULL, ...) {
+  if (!is_dm_keyed_tbl(y)) {
+    return(NextMethod())
+  }
+
+  if (is.null(by)) {
+    by <- keyed_by(x, y)
+  }
+
+  joined_tbl <- semi_join(
+    unclass_keyed_tbl(x), unclass_keyed_tbl(y), by,
+    copy = copy,
+    ...
+  )
+
+  new_keyed_tbl_from_keys_info(joined_tbl, keyed_get_info(x))
+}
+
 #' @export
 anti_join.dm <- function(x, ...) {
   check_zoomed(x)
@@ -455,13 +605,33 @@ anti_join.dm <- function(x, ...) {
 
 #' @rdname dplyr_join
 #' @export
-anti_join.zoomed_dm <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
+anti_join.dm_zoomed <- function(x, y, by = NULL, copy = NULL, suffix = NULL, select = NULL, ...) {
   y_name <- as_string(enexpr(y))
   join_data <- prepare_join(x, {{ y }}, by, {{ select }}, suffix, copy, disambiguate = FALSE)
   joined_tbl <- anti_join(join_data$x_tbl, join_data$y_tbl, join_data$by, copy = FALSE, ...)
   replace_zoomed_tbl(x, joined_tbl, join_data$new_col_names)
 }
 
+#' @rdname dplyr_join
+#' @export
+anti_join.dm_keyed_tbl <- function(x, y, by = NULL, copy = NULL, ...) {
+  if (!is_dm_keyed_tbl(y)) {
+    return(NextMethod())
+  }
+
+  if (is.null(by)) {
+    by <- keyed_by(x, y)
+  }
+
+  joined_tbl <- anti_join(
+    unclass_keyed_tbl(x), unclass_keyed_tbl(y), by,
+    copy = copy,
+    ...
+  )
+
+  new_keyed_tbl_from_keys_info(joined_tbl, keyed_get_info(x))
+}
+
 #' @export
 nest_join.dm <- function(x, ...) {
   check_zoomed(x)
@@ -470,7 +640,7 @@ nest_join.dm <- function(x, ...) {
 #' @rdname dplyr_join
 #' @inheritParams dplyr::nest_join
 #' @export
-nest_join.zoomed_dm <- function(x, y, by = NULL, copy = FALSE, keep = FALSE, name = NULL, ...) {
+nest_join.dm_zoomed <- function(x, y, by = NULL, copy = FALSE, keep = FALSE, name = NULL, ...) {
   y_name <- as_string(enexpr(y))
   name <- name %||% y_name
   join_data <- prepare_join(x, {{ y }}, by, NULL, NULL, NULL, disambiguate = FALSE)
