@@ -1,46 +1,3 @@
-# FIXME: #313: learn only from current source
-
-test_that("Standard learning from MSSQL (schema 'dbo') or Postgres (schema 'public') and get_src_tbl_names() works?", {
-  skip_if_schema_not_supported()
-
-  # dm_learn_from_mssql() --------------------------------------------------
-  con_db <- my_test_con()
-
-  # create an object on the MSSQL-DB that can be learned
-  withr::defer(
-    try(walk(
-      remote_tbl_names,
-      ~ try(dbExecute(con_db, paste0("DROP TABLE ", .x)))
-    ))
-  )
-
-  dm_for_filter_copied <- copy_dm_to(con_db, dm_for_filter(), temporary = FALSE, table_names = ~ DBI::SQL(unique_db_table_name(.x)))
-  order_of_deletion <- c("tf_2", "tf_1", "tf_5", "tf_6", "tf_4", "tf_3")
-
-  remote_tbl_names <-
-    map_chr(
-      dm_get_tables(dm_for_filter_copied)[order_of_deletion],
-      dbplyr::remote_name
-    ) %>%
-    SQL() %>%
-    DBI::dbUnquoteIdentifier(conn = con_db) %>%
-    map_chr(~ .x@name[["table"]])
-
-  remote_tbl_map <- set_names(remote_tbl_names, gsub("^(tf_.).*$", "\\1", remote_tbl_names))
-
-  # test 'get_src_tbl_names()'
-  src_tbl_names <- sort(unname(gsub("^.*\\.", "", get_src_tbl_names(con_db))))
-  expect_identical(
-    # fail if there are other tables in the default schema
-    src_tbl_names[grep("tf_._", src_tbl_names)],
-    sort(dbQuoteIdentifier(con_db, remote_tbl_names))
-  )
-
-  expect_snapshot({
-    dm_from_con(con_db)[integer()]
-  })
-})
-
 test_that("Standard learning from MSSQL (schema 'dbo') or Postgres (schema 'public') and get_src_tbl_names() works?", {
   skip_if_schema_not_supported()
 
@@ -213,11 +170,11 @@ test_that("'schema_if()' works", {
         schema = "schema",
         table = "table",
         con = con_db,
-        dbname = "db"
+        dbname = "database"
       ),
       "SQL"
     )),
-    "\"db\".\"schema\".\"table\"|`db`.`schema`.`table`"
+    "\"database\".\"schema\".\"table\"|`database`.`schema`.`table`"
   )
 
   # schema and table set
@@ -483,4 +440,9 @@ test_that("dm_from_con() with mariaDB", {
   expect_snapshot_output(my_dm <- dm_from_con(my_db))
   expect_snapshot(dm::dm_get_all_fks(my_dm))
   expect_snapshot(dm::dm_get_all_pks(my_dm))
+
+  # multiple schemata work
+  expect_snapshot_output(my_dm <- dm_from_con(my_db, schema = c("Accidents", "Ad")))
+  expect_snapshot(dm::dm_get_all_fks(my_dm))
+  expect_snapshot(dm::dm_get_all_pks(my_dm))
 })
