@@ -6,11 +6,11 @@ test_that("path argument", {
   expect_identical(readLines(path), c("dm::dm(", ")"))
 })
 
-local_options(lifecycle_verbosity = "warning")
-
 test_that("output", {
   skip_if_not_installed("nycflights13")
 
+  local_options(lifecycle_verbosity = "warning")
+
   expect_snapshot({
     "empty"
     empty_dm() %>% dm_paste()
@@ -83,3 +83,18 @@ test_that("output", {
     ))
   })
 })
+
+test_that("output 2", {
+  skip_if(getRversion() < "4.2")
+  local_options(lifecycle_verbosity = "warning")
+
+  expect_snapshot({
+    "no error for factor column that leads to code with width > 500"
+    dm(tibble(a = factor(levels = expand.grid(
+      letters, as.character(1:5)
+    ) %>%
+      transmute(x = paste0(Var1, Var2)) %>%
+      pull()))) %>%
+      dm_paste(options = "tables")
+  })
+})
