@@ -1,10 +1,15 @@
+default_eval <- function() {
+  # Only build vignettes in pkgdown for now
+  (Sys.getenv("IN_PKGDOWN") != "") || grepl("^cran-", Sys.getenv("GITHUB_HEAD_REF"))
+}
+
 knitr::opts_chunk$set(
   error = (Sys.getenv("IN_PKGDOWN") == "") || !dm:::dm_has_financial(),
   # Only build vignettes:
   # - on pkgdown
   # - if not running in GitHub Actions
   # - if running in a branch that starts with CRAN in GitHub Actions
-  eval = (Sys.getenv("IN_PKGDOWN") != "") || (Sys.getenv("GITHUB_WORKFLOW") == "") || grepl("^cran-", Sys.getenv("GITHUB_HEAD_REF")),
+  eval = default_eval(),
   message = TRUE,
   collapse = TRUE,
   comment = "#>"
@@ -28,7 +33,7 @@ if (rlang::has_length(grep('^library[(]"?tidyverse"?[)]', input))) {
   library(tidyverse)
 }
 if (rlang::has_length(grep('^library[(]"?dplyr"?[)]', input))) {
-  library(tidyverse)
+  library(dplyr)
 }
 
 ## Link helper to enable links only on pkgdown
